@@ -15,8 +15,8 @@
 """Trains a halfcheetah to run in the +x direction."""
 
 import brax
+from brax import jumpy as jp
 from brax.envs import env
-import jax.numpy as jnp
 
 
 class Halfcheetah(env.Env):
@@ -25,15 +25,23 @@ class Halfcheetah(env.Env):
   def __init__(self, **kwargs):
     super().__init__(_SYSTEM_CONFIG, **kwargs)
 
-  def reset(self, rng: jnp.ndarray) -> env.State:
+  def reset(self, rng: jp.ndarray) -> env.State:
     """Resets the environment to an initial state."""
-    qp = self.sys.default_qp()
+    rng, rng1, rng2 = jp.random_split(rng, 3)
+    qpos = self.sys.default_angle() + jp.random_uniform(
+        rng1, (self.sys.num_joint_dof,), -.1, .1)
+    qvel = jp.random_uniform(rng2, (self.sys.num_joint_dof,), -.1, .1)
+    qp = self.sys.default_qp(joint_angle=qpos, joint_velocity=qvel)
     info = self.sys.info(qp)
     obs = self._get_obs(qp, info)
-    reward, done = jnp.zeros(2)
-    return env.State(qp, obs, reward, done)
+    reward, done, zero = jp.zeros(3)
+    metrics = {
+        'reward_ctrl_cost': zero,
+        'reward_forward': zero,
+    }
+    return env.State(qp, obs, reward, done, metrics)
 
-  def step(self, state: env.State, action: jnp.ndarray) -> env.State:
+  def step(self, state: env.State, action: jp.ndarray) -> env.State:
     """Run one timestep of the environment's dynamics."""
     qp, info = self.sys.step(state.qp, action)
     obs = self._get_obs(qp, info)
@@ -41,12 +49,14 @@ class Halfcheetah(env.Env):
     x_before = state.qp.pos[0, 0]
     x_after = qp.pos[0, 0]
     forward_reward = (x_after - x_before) / self.sys.config.dt
-    ctrl_cost = -.1 * jnp.sum(jnp.square(action))
+    ctrl_cost = -.1 * jp.sum(jp.square(action))
     reward = forward_reward + ctrl_cost
+    state.metrics.update(
+        reward_ctrl_cost=ctrl_cost, reward_forward=forward_reward)
 
     return state.replace(qp=qp, obs=obs, reward=reward)
 
-  def _get_obs(self, qp: brax.QP, info: brax.Info) -> jnp.ndarray:
+  def _get_obs(self, qp: brax.QP, info: brax.Info) -> jp.ndarray:
     """Observe halfcheetah body position and velocities."""
     # some pre-processing to pull joint angles and velocities
     (joint_angle,), (joint_vel,) = self.sys.joints[0].angle_vel(qp)
@@ -63,7 +73,7 @@ class Halfcheetah(env.Env):
     # joint angle velocities (8,)
     qvel = [qp.vel[0], qp.ang[0], joint_vel]
 
-    return jnp.concatenate(qpos + qvel)
+    return jp.concatenate(qpos + qvel)
 
 _SYSTEM_CONFIG = """
 bodies {
@@ -415,7 +425,7 @@ actuators {
   torque {
   }
 }
-friction: 0.4000000059604645
+friction: 0.6000000059604645
 gravity {
   z: -9.8100004196167
 }
@@ -441,6 +451,18 @@ collide_include {
   first: "floor"
   second: "fthigh"
 }
+collide_include {
+  first: "floor"
+  second: "bshin"
+}
+collide_include {
+  first: "floor"
+  second: "fshin"
+}
+collide_include {
+  first: "bfoot"
+  second: "ffoot"
+}
 dt: 0.05
 substeps: 16
 frozen {
