@@ -18,13 +18,12 @@
 import abc
 from typing import Any, List, Tuple
 
+from brax import jumpy as jp
+from brax import math
+from brax import pytree
 from brax.physics import bodies
 from brax.physics import config_pb2
-from brax.physics import math
-from brax.physics import pytree
-from brax.physics.base import P, QP, euler_to_quat, take, vec_to_np
-import jax
-import jax.numpy as jnp
+from brax.physics.base import P, QP, vec_to_arr
 
 
 class Joint(abc.ABC):
@@ -47,31 +46,33 @@ class Joint(abc.ABC):
       body: batched body that contain the parents and children of each joint
       spring_damping_coeff: coefficient for setting default spring damping
     """
-    self.stiffness = jnp.array([j.stiffness for j in joints])
-    self.angular_damping = jnp.array([j.angular_damping for j in joints])
-    self.spring_damping = jnp.array([
+    self.stiffness = jp.array([j.stiffness for j in joints])
+    self.angular_damping = jp.array([j.angular_damping for j in joints])
+    self.spring_damping = jp.array([
         j.spring_damping if j.HasField('spring_damping') else
-        spring_damping_coeff * jnp.sqrt(j.stiffness) for j in joints
+        spring_damping_coeff * jp.sqrt(j.stiffness) for j in joints
     ])
-    self.limit_strength = jnp.array([
+    self.limit_strength = jp.array([
         j.limit_strength if j.HasField('limit_strength') else j.stiffness
         for j in joints
     ])
-    self.limit = jnp.array([[[i.min, i.max]
-                             for i in j.angle_limit]
-                            for j in joints]) / 180.0 * jnp.pi
-    self.body_p = take(body, jnp.array([body.index[j.parent] for j in joints]))
-    self.body_c = take(body, jnp.array([body.index[j.child] for j in joints]))
-    self.off_p = jnp.array([vec_to_np(j.parent_offset) for j in joints])
-    self.off_c = jnp.array([vec_to_np(j.child_offset) for j in joints])
+    self.limit = jp.array([[[i.min, i.max]
+                            for i in j.angle_limit]
+                           for j in joints]) / 180.0 * jp.pi
+    self.body_p = jp.take(body, [body.index[j.parent] for j in joints])
+    self.body_c = jp.take(body, [body.index[j.child] for j in joints])
+    self.off_p = jp.array([vec_to_arr(j.parent_offset) for j in joints])
+    self.off_c = jp.array([vec_to_arr(j.child_offset) for j in joints])
     self.index = {j.name: i for i, j in enumerate(joints)}
     self.dof = len(joints[0].angle_limit)
-    v_rot = jax.vmap(math.rotate, in_axes=[0, None])
-    relative_quats = jnp.array(
-        [euler_to_quat(j.reference_rotation) for j in joints])
-    self.axis_c = jnp.array(
-        [v_rot(jnp.eye(3), euler_to_quat(j.rotation)) for j in joints])
-    self.axis_p = jnp.array(
+    v_rot = jp.vmap(math.rotate, include=[True, False])
+    relative_quats = jp.array(
+        [math.euler_to_quat(vec_to_arr(j.reference_rotation)) for j in joints])
+    self.axis_c = jp.array([
+        v_rot(jp.eye(3), math.euler_to_quat(vec_to_arr(j.rotation)))
+        for j in joints
+    ])
+    self.axis_p = jp.array(
         [v_rot(j, r) for j, r in zip(self.axis_c, relative_quats)])
 
   def apply(self, qp: QP) -> P:
@@ -83,16 +84,16 @@ class Joint(abc.ABC):
     Returns:
       dP: Impulses on all bodies to maintain joint constraints
     """
-    qp_p = take(qp, self.body_p.idx)
-    qp_c = take(qp, self.body_c.idx)
-    dp_p, dp_c = self.apply_reduced(qp_p, qp_c)
+    qp_p = jp.take(qp, self.body_p.idx)
+    qp_c = jp.take(qp, self.body_c.idx)
+    dp_p, dp_c = jp.vmap(type(self).apply_reduced)(self, qp_p, qp_c)
 
     # sum together all impulse contributions across parents and children
-    body_idx = jnp.concatenate((self.body_p.idx, self.body_c.idx))
-    dp_vel = jnp.concatenate((dp_p.vel, dp_c.vel))
-    dp_ang = jnp.concatenate((dp_p.ang, dp_c.ang))
-    dp_vel = jax.ops.segment_sum(dp_vel, body_idx, qp.pos.shape[0])
-    dp_ang = jax.ops.segment_sum(dp_ang, body_idx, qp.pos.shape[0])
+    body_idx = jp.concatenate((self.body_p.idx, self.body_c.idx))
+    dp_vel = jp.concatenate((dp_p.vel, dp_c.vel))
+    dp_ang = jp.concatenate((dp_p.ang, dp_c.ang))
+    dp_vel = jp.segment_sum(dp_vel, body_idx, qp.pos.shape[0])
+    dp_ang = jp.segment_sum(dp_ang, body_idx, qp.pos.shape[0])
 
     return P(vel=dp_vel, ang=dp_ang)
 
@@ -107,14 +108,14 @@ class Joint(abc.ABC):
       vel: n-tuple of joint velocities where n = # DoF of the joint
     """
 
-    @jax.vmap
+    @jp.vmap
     def op(joint, qp_p, qp_c):
       axes, angles = joint.axis_angle(qp_p, qp_c)
-      vels = tuple([jnp.dot(qp_p.ang - qp_c.ang, axis) for axis in axes])
+      vels = tuple([jp.dot(qp_p.ang - qp_c.ang, axis) for axis in axes])
       return angles, vels
 
-    qp_p = take(qp, self.body_p.idx)
-    qp_c = take(qp, self.body_c.idx)
+    qp_p = jp.take(qp, self.body_p.idx)
+    qp_c = jp.take(qp, self.body_c.idx)
     angles, vels = op(self, qp_p, qp_c)
 
     return angles, vels
@@ -163,11 +164,10 @@ class Revolute(Joint):
   def __init__(self, joints: List[config_pb2.Joint], body: bodies.Body):
     super().__init__(joints, body, .5)
 
-  @jax.vmap
   def apply_reduced(self, qp_p: QP, qp_c: QP) -> Tuple[P, P]:
     """Returns calculated impulses in compressed joint space."""
-    pos_p, vel_p = math.to_world(qp_p, self.off_p)
-    pos_c, vel_c = math.to_world(qp_c, self.off_c)
+    pos_p, vel_p = qp_p.to_world(self.off_p)
+    pos_c, vel_c = qp_c.to_world(self.off_c)
 
     # push the bodies towards their offsets
     # TODO: cap this damping so that it can't overcorrect
@@ -179,18 +179,18 @@ class Revolute(Joint):
     # torque the bodies to align their axes
     (axis,), (angle,) = self.axis_angle(qp_p, qp_c)
     axis_c = math.rotate(self.axis_c[0], qp_c.rot)
-    torque = self.stiffness * jnp.cross(axis, axis_c)
+    torque = self.stiffness * jp.cross(axis, axis_c)
 
     # torque the bodies to stay within angle limits
-    dang = jnp.where(angle < self.limit[0][0], self.limit[0][0] - angle, 0)
-    dang = jnp.where(angle > self.limit[0][1], self.limit[0][1] - angle, dang)
+    dang = jp.where(angle < self.limit[0][0], self.limit[0][0] - angle, 0)
+    dang = jp.where(angle > self.limit[0][1], self.limit[0][1] - angle, dang)
     torque -= self.limit_strength * axis * dang
 
     # damp the angular motion
     torque -= self.angular_damping * (qp_p.ang - qp_c.ang)
 
-    dang_p = jnp.matmul(self.body_p.inertia, torque)
-    dang_c = jnp.matmul(self.body_c.inertia, -torque)
+    dang_p = jp.matmul(self.body_p.inertia, torque)
+    dang_c = jp.matmul(self.body_c.inertia, -torque)
     dp_p = dp_p.replace(ang=dp_p.ang + dang_p)
     dp_c = dp_c.replace(ang=dp_c.ang + dang_c)
 
@@ -215,11 +215,10 @@ class Universal(Joint):
   of a spherical joint.  See `Spherical` for details.
   """
 
-  @jax.vmap
   def apply_reduced(self, qp_p: QP, qp_c: QP) -> Tuple[P, P]:
     """Returns calculated impulses in compressed joint space."""
-    pos_p, vel_p = math.to_world(qp_p, self.off_p)
-    pos_c, vel_c = math.to_world(qp_c, self.off_c)
+    pos_p, vel_p = qp_p.to_world(self.off_p)
+    pos_c, vel_c = qp_c.to_world(self.off_c)
 
     # push the bodies towards their offsets
     # TODO: cap this damping so that it can't overcorrect
@@ -230,23 +229,23 @@ class Universal(Joint):
 
     # torque the bodies to align to a joint plane
     (axis_1, axis_2), (angle_1, angle_2) = self.axis_angle(qp_p, qp_c)
-    axis_c_proj = axis_2 - jnp.dot(axis_2, axis_1) * axis_1
-    axis_c_proj = axis_c_proj / math.safe_norm(axis_c_proj)
-    torque = (self.limit_strength / 5.) * jnp.cross(axis_c_proj, axis_2)
+    axis_c_proj = axis_2 - jp.dot(axis_2, axis_1) * axis_1
+    axis_c_proj = axis_c_proj / jp.safe_norm(axis_c_proj)
+    torque = (self.limit_strength / 5.) * jp.cross(axis_c_proj, axis_2)
 
     # torque the bodies to stay within angle limits
     limit_1, limit_2 = self.limit
-    dang_1 = jnp.where(angle_1 < limit_1[0], limit_1[0] - angle_1, 0)
-    dang_1 = jnp.where(angle_1 > limit_1[1], limit_1[1] - angle_1, dang_1)
-    dang_2 = jnp.where(angle_2 < limit_2[0], limit_2[0] - angle_2, 0)
-    dang_2 = jnp.where(angle_2 > limit_2[1], limit_2[1] - angle_2, dang_2)
+    dang_1 = jp.where(angle_1 < limit_1[0], limit_1[0] - angle_1, 0)
+    dang_1 = jp.where(angle_1 > limit_1[1], limit_1[1] - angle_1, dang_1)
+    dang_2 = jp.where(angle_2 < limit_2[0], limit_2[0] - angle_2, 0)
+    dang_2 = jp.where(angle_2 > limit_2[1], limit_2[1] - angle_2, dang_2)
     torque -= self.limit_strength * (axis_1 * dang_1 + axis_2 * dang_2)
 
     # damp the angular motion
     torque -= self.angular_damping * (qp_p.ang - qp_c.ang)
 
-    dang_p = jnp.matmul(self.body_p.inertia, torque)
-    dang_c = jnp.matmul(self.body_c.inertia, -torque)
+    dang_p = jp.matmul(self.body_p.inertia, torque)
+    dang_c = jp.matmul(self.body_c.inertia, -torque)
     dp_p = dp_p.replace(ang=dp_p.ang + dang_p)
     dp_c = dp_c.replace(ang=dp_c.ang + dang_c)
 
@@ -254,7 +253,7 @@ class Universal(Joint):
 
   def axis_angle(self, qp_p: QP, qp_c: QP) -> Tuple[Any, Any]:
     """Returns axes and angles of a single joint."""
-    v_rot = jax.vmap(math.rotate, in_axes=[0, None])
+    v_rot = jp.vmap(math.rotate, include=[True, False])
     axis_p_rotated = v_rot(self.axis_p, qp_p.rot)
     axis_c_rotated = v_rot(self.axis_c, qp_c.rot)
     axis_1_p = axis_p_rotated[0]
@@ -263,20 +262,16 @@ class Universal(Joint):
     axis_2_c = axis_c_rotated[1]
     axis_3_c = axis_c_rotated[2]
 
-    line_of_nodes = jnp.cross(axis_3_c, axis_1_p)
-    line_of_nodes = line_of_nodes / (1e-10 + math.safe_norm(line_of_nodes))
-
-    y_N_normal = axis_1_p
-
-    psi = math.signed_angle(y_N_normal, axis_2_p, line_of_nodes)
-
-    axis_1_p_in_xz_c = jnp.dot(axis_1_p, axis_1_c) * axis_1_c + jnp.dot(
+    line_of_nodes = jp.cross(axis_3_c, axis_1_p)
+    line_of_nodes = line_of_nodes / (1e-10 + jp.safe_norm(line_of_nodes))
+    y_n_normal = axis_1_p
+    psi = math.signed_angle(y_n_normal, axis_2_p, line_of_nodes)
+    axis_1_p_in_xz_c = jp.dot(axis_1_p, axis_1_c) * axis_1_c + jp.dot(
         axis_1_p, axis_2_c) * axis_2_c
     axis_1_p_in_xz_c = axis_1_p_in_xz_c / (1e-10 +
-                                           math.safe_norm(axis_1_p_in_xz_c))
-    theta = jnp.arccos(jnp.clip(jnp.dot(axis_1_p_in_xz_c, axis_1_p), -1,
-                                1)) * jnp.sign(jnp.dot(axis_1_p, axis_3_c))
-
+                                           jp.safe_norm(axis_1_p_in_xz_c))
+    theta = jp.arccos(jp.clip(jp.dot(axis_1_p_in_xz_c, axis_1_p), -1,
+                              1)) * jp.sign(jp.dot(axis_1_p, axis_3_c))
     axis = (axis_1_p, axis_2_c)
     angle = (psi, theta)
 
@@ -291,13 +286,12 @@ class Spherical(Joint):
     x-y'-z'' convention between the parent and child.  Uses the line of nodes
     construction described in section 3.2.3.2 here:
     https://www.sedris.org/wg8home/Documents/WG80485.pdf
-    """
+  """
 
-  @jax.vmap
   def apply_reduced(self, qp_p: QP, qp_c: QP) -> Tuple[P, P]:
     """Returns calculated impulses in compressed joint space."""
-    pos_p, vel_p = math.to_world(qp_p, self.off_p)
-    pos_c, vel_c = math.to_world(qp_c, self.off_c)
+    pos_p, vel_p = qp_p.to_world(self.off_p)
+    pos_c, vel_c = qp_c.to_world(self.off_c)
 
     # push the bodies towards their offsets
     # TODO: cap this damping so that it can't overcorrect
@@ -310,12 +304,12 @@ class Spherical(Joint):
     axis, angle = self.axis_angle(qp_p, qp_c)
     angle_1, angle_2, angle_3 = angle
     limit_1, limit_2, limit_3 = self.limit
-    dang_1 = jnp.where(angle_1 < limit_1[0], limit_1[0] - angle_1, 0)
-    dang_1 = jnp.where(angle_1 > limit_1[1], limit_1[1] - angle_1, dang_1)
-    dang_2 = jnp.where(angle_2 < limit_2[0], limit_2[0] - angle_2, 0)
-    dang_2 = jnp.where(angle_2 > limit_2[1], limit_2[1] - angle_2, dang_2)
-    dang_3 = jnp.where(angle_3 < limit_3[0], limit_3[0] - angle_3, 0)
-    dang_3 = jnp.where(angle_3 > limit_3[1], limit_3[1] - angle_3, dang_3)
+    dang_1 = jp.where(angle_1 < limit_1[0], limit_1[0] - angle_1, 0)
+    dang_1 = jp.where(angle_1 > limit_1[1], limit_1[1] - angle_1, dang_1)
+    dang_2 = jp.where(angle_2 < limit_2[0], limit_2[0] - angle_2, 0)
+    dang_2 = jp.where(angle_2 > limit_2[1], limit_2[1] - angle_2, dang_2)
+    dang_3 = jp.where(angle_3 < limit_3[0], limit_3[0] - angle_3, 0)
+    dang_3 = jp.where(angle_3 > limit_3[1], limit_3[1] - angle_3, dang_3)
 
     # TODO: fully decouple different torque axes
     torque = axis[0] * dang_1 + axis[1] * dang_2 + axis[2] * dang_3
@@ -324,14 +318,14 @@ class Spherical(Joint):
     # damp the angular motion
     torque -= self.angular_damping * (qp_p.ang - qp_c.ang)
 
-    dp_p = dp_p.replace(ang=dp_p.ang + jnp.matmul(self.body_p.inertia, torque))
-    dp_c = dp_c.replace(ang=dp_c.ang + jnp.matmul(self.body_c.inertia, -torque))
+    dp_p = dp_p.replace(ang=dp_p.ang + jp.matmul(self.body_p.inertia, torque))
+    dp_c = dp_c.replace(ang=dp_c.ang + jp.matmul(self.body_c.inertia, -torque))
 
     return dp_p, dp_c
 
   def axis_angle(self, qp_p: QP, qp_c: QP) -> Tuple[Any, Any]:
     """Returns axes and angles of a single joint."""
-    v_rot = jax.vmap(math.rotate, in_axes=[0, None])
+    v_rot = jp.vmap(math.rotate, include=[True, False])
     axis_p_rotated = v_rot(self.axis_p, qp_p.rot)
     axis_c_rotated = v_rot(self.axis_c, qp_c.rot)
     axis_1_p = axis_p_rotated[0]
@@ -340,25 +334,19 @@ class Spherical(Joint):
     axis_2_c = axis_c_rotated[1]
     axis_3_c = axis_c_rotated[2]
 
-    line_of_nodes = jnp.cross(axis_3_c, axis_1_p)
-    line_of_nodes = line_of_nodes / (1e-10 + math.safe_norm(line_of_nodes))
-
-    y_N_normal = axis_1_p
-
-    psi = math.signed_angle(y_N_normal, axis_2_p, line_of_nodes)
-
-    axis_1_p_in_xz_c = jnp.dot(axis_1_p, axis_1_c) * axis_1_c + jnp.dot(
+    line_of_nodes = jp.cross(axis_3_c, axis_1_p)
+    line_of_nodes = line_of_nodes / (1e-10 + jp.safe_norm(line_of_nodes))
+    y_n_normal = axis_1_p
+    psi = math.signed_angle(y_n_normal, axis_2_p, line_of_nodes)
+    axis_1_p_in_xz_c = jp.dot(axis_1_p, axis_1_c) * axis_1_c + jp.dot(
         axis_1_p, axis_2_c) * axis_2_c
     axis_1_p_in_xz_c = axis_1_p_in_xz_c / (1e-10 +
-                                           math.safe_norm(axis_1_p_in_xz_c))
-
-    ang_between_1_p_xz_c = jnp.dot(axis_1_p_in_xz_c, axis_1_p)
-    theta = jnp.arccos(jnp.clip(ang_between_1_p_xz_c, -1, 1)) * jnp.sign(
-        jnp.dot(axis_1_p, axis_3_c))
-    yc_N_normal = -1. * axis_3_c
-
-    phi = math.signed_angle(yc_N_normal, axis_2_c, line_of_nodes)
-
+                                           jp.safe_norm(axis_1_p_in_xz_c))
+    ang_between_1_p_xz_c = jp.dot(axis_1_p_in_xz_c, axis_1_p)
+    theta = jp.arccos(jp.clip(ang_between_1_p_xz_c, -1, 1)) * jp.sign(
+        jp.dot(axis_1_p, axis_3_c))
+    yc_n_normal = -axis_3_c
+    phi = math.signed_angle(yc_n_normal, axis_2_c, line_of_nodes)
     axis = (axis_1_p, axis_2_c, axis_3_c)
     angle = (psi, theta, phi)
 
